package provide FR_comboBox 0.1

namespace eval ::FR_comboBox \
{
 namespace export cb
}

proc ::FR_comboBox::cb {args} \
{
 #----------the usage of FR_comboBox:-----------
 #
 # ::FR_comboBox::cb pathName ?options?
 #
 # Options:
 #
 # in the case of creation:
 # -variable {the textvariable of the entry-widget}
 # -command {a procedure which is called when a item from the listbox is selected}
 #
 # in the case of creation and/or filling:
 # -listbox_content {the content of your popup listbox}
 # -default_value {the value which is showed in the entry-box without a choice in the popup listbox}
 # -entry_width {if you wish a fixed width of the entry-box in count of letters or set it to "auto";
 #               in the "auto" case the width of the the entry-box is depending on the content of
 #               the listbox (limited to 50 characters)}
 #
 # in the case configuration: 
 # -conf_widget {entry | button | listbox}
 # -conf_option {the same options as for a normal tk-widget specified with "-conf_widget"}
 #
 # to change the status:
 # -state {normal | disable}
 #
 # to read out the content of the entry widget:
 # -read
 
 variable FR_cb_vars; variable FR_cb_path; 
 
 
 #---we read the pathname
 set FR_cb_path ""
 set FR_cb_path [lindex $args 0]


 #---we check the correctness of the pathname
 if {$FR_cb_path eq ""} {error "You must hand over a valid pathname, not an empty string"}
  
 set FR_cb_path [string trimleft $FR_cb_path .]
 set FR_cb_vars($FR_cb_path,PATH) $FR_cb_path

  
 #---presetting of the ENTRY_WIDTH
 set FR_cb_vars($FR_cb_path,EXIST) 0
 
 if {[winfo exists .$FR_cb_vars($FR_cb_path,PATH).cb_entry]} \
 {
  set FR_cb_vars($FR_cb_path,ENTRY_WIDTH) [.$FR_cb_vars($FR_cb_path,PATH).cb_entry cget -width]
  #---if the combobox already exists we set EXIST to 1
  set FR_cb_vars($FR_cb_path,EXIST) 1
 } else \
 {
  set FR_cb_vars($FR_cb_path,ENTRY_WIDTH) 2
 }


 #---default values or presettings
 set FR_cb_vars($FR_cb_path,LB_STATUS) 0
 
 foreach ARG [list ENTRY_WIDTH_SET LB_CONTENT_SET LB_DEFAULT_SET VARIABLE_SET COMMAND_SET PARENT_SET CONF_WIDGET_SET CONF_OPTION_SET STATE_SET READ_SET] \
 {
  set FR_cb_vars($FR_cb_path,$ARG) 0
 }


 #---we check the args 
 for {set i 1} {$i < [llength $args]} {incr i 2} \
  {
   set arg [lindex $args $i]

   switch -glob -- $arg \
    {-entry_width     {set FR_cb_vars($FR_cb_path,ENTRY_WIDTH) [lindex $args [expr {$i+1}]]; set FR_cb_vars($FR_cb_path,ENTRY_WIDTH_SET) 1} \
     -listbox_content {set FR_cb_vars($FR_cb_path,LB_CONTENT) [lindex $args [expr {$i+1}]]; set FR_cb_vars($FR_cb_path,LB_CONTENT_SET) 1} \
     -default_value   {set FR_cb_vars($FR_cb_path,LB_DEFAULT) [lindex $args [expr {$i+1}]]; set FR_cb_vars($FR_cb_path,LB_DEFAULT_SET) 1} \
     -variable        {set FR_cb_vars($FR_cb_path,VARIABLE) [lindex $args [expr {$i+1}]]; set FR_cb_vars($FR_cb_path,VARIABLE_SET) 1} \
     -command         {set FR_cb_vars($FR_cb_path,COMMAND) [lindex $args [expr {$i+1}]]; set FR_cb_vars($FR_cb_path,COMMAND_SET) 1} \
     -parent          {set FR_cb_vars($FR_cb_path,PARENT) [lindex $args [expr {$i+1}]]; set FR_cb_vars($FR_cb_path,PARENT_SET) 1} \
     -conf_widget     {set FR_cb_vars($FR_cb_path,CONF_WIDGET) [lindex $args [expr {$i+1}]]; set FR_cb_vars($FR_cb_path,CONF_WIDGET_SET) 1} \
     -conf_option     {set FR_cb_vars($FR_cb_path,CONF_OPTION) [lindex $args [expr {$i+1}]]; set FR_cb_vars($FR_cb_path,CONF_OPTION_SET) 1} \
     -state           {set FR_cb_vars($FR_cb_path,STATE) [lindex $args [expr {$i+1}]]; set FR_cb_vars($FR_cb_path,STATE_SET) 1} \
     -read            {set FR_cb_vars($FR_cb_path,READ_SET) 1} \
     --               {break} \
     *                {error "unknown option: $arg"}}
     
   }

   
 #---put a content in the entry widget
 proc ::FR_comboBox::cb_insert_entry {widget value} \
 {
  .$widget.cb_entry configure -state normal
  .$widget.cb_entry delete 0 end
  .$widget.cb_entry insert end $value
  .$widget.cb_entry configure -state disable
 }    


 #---configure a combobox
 proc ::FR_comboBox::FR_comboBox_config {} \
 {
  variable FR_cb_vars; variable FR_cb_path
   
  if { ![expr {$FR_cb_vars($FR_cb_path,CONF_WIDGET) eq "entry" || $FR_cb_vars($FR_cb_path,CONF_WIDGET) eq "button" || $FR_cb_vars($FR_cb_path,CONF_WIDGET) eq "listbox"}] } \
  {error "Wrong widget choiced: $FR_cb_vars($FR_cb_path,CONF_WIDGET). Please use \"entry\", \"button\" or \"listbox\""}
   
  if {$FR_cb_vars($FR_cb_path,CONF_WIDGET) eq "entry"} {set PATH $FR_cb_vars($FR_cb_path,PATH).cb_entry} 
  if {$FR_cb_vars($FR_cb_path,CONF_WIDGET) eq "button"} {set PATH $FR_cb_vars($FR_cb_path,PATH).cb_button}
  if {$FR_cb_vars($FR_cb_path,CONF_WIDGET) eq "listbox"} {set PATH $FR_cb_vars($FR_cb_path,PATH).lb.cb_listbox}
    
  if {![winfo exists .$PATH]} {error "The $FR_cb_vars($FR_cb_path,CONF_WIDGET)-widget: \".$PATH\" doesn't exists."}
    
  set CATCHRETURN [catch {eval .$PATH configure $FR_cb_vars($FR_cb_path,CONF_OPTION)} CATCHOUTPUT]
   
  if {$CATCHRETURN != 0} {error "Can't make this configuration. The reason is: $CATCHOUTPUT"}
 }

 
 #---read a combobox
 proc ::FR_comboBox::FR_comboBox_read {widget} \
 {    
  return [.${widget}.cb_entry get]
 }

  
 #---remove/add the x or y scrollbars
 proc ::FR_comboBox::scrollbar_manager {widget ORIENTATION ACTION} \
 {
  if {$ORIENTATION eq "y"} \
  {
   set WIDGET $widget.cb_yscrollbar
   set gridcmd "grid .$widget.cb_yscrollbar -row 0 -column 1 -sticky ns"
  }

  if {$ORIENTATION eq "x"} \
  {
   set WIDGET $widget.cb_xscrollbar
   set gridcmd "grid .$widget.cb_xscrollbar -row 1 -column 0 -sticky ew -columnspan 2"
  }

  if { $ACTION eq "remove" } \
  {
   if {[winfo ismapped .$widget]} {grid remove .$WIDGET}
  }
 
  if { $ACTION eq "view" } \
  {
   if {![winfo ismapped .$widget]} {eval $gridcmd}
  }
 }


 #---we fill the listbox
 proc ::FR_comboBox::fill_lb {} \
 {
  variable FR_cb_vars; variable FR_cb_path
      
  set FR_cb_vars($FR_cb_path,LB_PATH) "$FR_cb_vars($FR_cb_path,PATH).lb"
    
  if {![winfo exists .$FR_cb_vars($FR_cb_path,LB_PATH)]} {error "The \"$FR_cb_vars($FR_cb_path,LB_PATH)\"-widget doesn't exists."}
  
  #---first we delete the old content
  .$FR_cb_vars($FR_cb_path,LB_PATH).cb_listbox delete 0 end
  
  #---now we fill the listbox with new content
  set MAX 2; # we need MAX for the width of the listbox
  set LISTLENGHT [llength $FR_cb_vars($FR_cb_path,LB_CONTENT)]
  
  if { $LISTLENGHT >= 1 } \
  {
   foreach arg $FR_cb_vars($FR_cb_path,LB_CONTENT) \
   {
    .$FR_cb_vars($FR_cb_path,LB_PATH).cb_listbox insert end $arg
    set COUNT [string length $arg]
    if {$COUNT > $MAX} {set MAX $COUNT}
   }
   
   #---for count of list elements < 8 we adapt the height of the listbox, in the other case we set the height to 8
   if { $LISTLENGHT <= 8} \
   {
    .$FR_cb_vars($FR_cb_path,LB_PATH).cb_listbox configure -height $LISTLENGHT
       
    ::FR_comboBox::scrollbar_manager $FR_cb_vars($FR_cb_path,LB_PATH) y remove
   } else \
   {
    .$FR_cb_vars($FR_cb_path,LB_PATH).cb_listbox configure -height 8
    ::FR_comboBox::scrollbar_manager $FR_cb_vars($FR_cb_path,LB_PATH) y view
    }
   }
   
   #---the auto-entry-width function, ENTRY_WIDTH_AUTO comes from the creation of the combobox
   if {$FR_cb_vars($FR_cb_path,ENTRY_WIDTH) eq "auto" || $FR_cb_vars($FR_cb_path,ENTRY_WIDTH_AUTO)} \
   {
    #---we set the entry width to the maximum count of letters and/or limit it to 50 letters
    if {$MAX < 50} {set FR_cb_vars($FR_cb_path,ENTRY_WIDTH) $MAX} else {set FR_cb_vars($FR_cb_path,ENTRY_WIDTH) 50}  
    set FR_cb_vars($FR_cb_path,ENTRY_WIDTH_SET) 1
   }
   
   #----we view or remove the x-axis scrollbar
   if { $MAX <= $FR_cb_vars($FR_cb_path,ENTRY_WIDTH)} \
   {
    ::FR_comboBox::scrollbar_manager $FR_cb_vars($FR_cb_path,LB_PATH) x remove
   } else \
   {::FR_comboBox::scrollbar_manager $FR_cb_vars($FR_cb_path,LB_PATH) x view}
  
   #---we set the marked element in the listbox to the LB_DEFAULT or to the first element in the listbox-content
   set SEARCHRESULT "-1"
   
   if {$FR_cb_vars($FR_cb_path,LB_DEFAULT_SET)} \
   {
    set SEARCHRESULT [lsearch -exact $FR_cb_vars($FR_cb_path,LB_CONTENT) $FR_cb_vars($FR_cb_path,LB_DEFAULT)]
   }
   
   if { $SEARCHRESULT == -1} {set SEARCHRESULT 0}
  
   .$FR_cb_vars($FR_cb_path,LB_PATH).cb_listbox selection set $SEARCHRESULT
   .$FR_cb_vars($FR_cb_path,LB_PATH).cb_listbox see $SEARCHRESULT
  
   ::FR_comboBox::cb_insert_entry $FR_cb_vars($FR_cb_path,PATH) [lindex $FR_cb_vars($FR_cb_path,LB_CONTENT) $SEARCHRESULT]
    
    
   if {$FR_cb_vars($FR_cb_path,ENTRY_WIDTH_SET)} \
   {.$FR_cb_vars($FR_cb_path,PATH).cb_entry configure -width $FR_cb_vars($FR_cb_path,ENTRY_WIDTH)}
 }
 
 
   
 #---create a combobox
 proc ::FR_comboBox::create {} \
 {
  variable FR_cb_vars; variable FR_cb_path
  variable FR_cb_image1; variable FR_cb_configure
  
  #---set the default for the -variable option
  if {$FR_cb_vars($FR_cb_path,VARIABLE_SET) != 1} {set FR_cb_vars($FR_cb_path,VARIABLE) "noname"}
  
  #---we check if the "-variable" parameter a array and make the arrayname to a global variable
  if {[regexp {([^(]*)([(]+)([^)]*)([)]+)} $FR_cb_vars($FR_cb_path,VARIABLE) all ARRAYNAME all TAIL all]} \
  {
   global [set ARRAYNAME]
   
   unset all TAIL
  } else \
  {
   #---if the "-variable" parameter a normal variable we make it to a global variable
   global [set FR_cb_vars($FR_cb_path,VARIABLE)]
  }
  
  #upvar #0 $FR_cb_vars($FR_cb_path,VARIABLE) VARIABLE
  
  if {[winfo exists .$FR_cb_vars($FR_cb_path,PATH)]} \
  {error "The widget with the path \".$FR_cb_vars($FR_cb_path,PATH)\" already exits. Please choice an other name."}
    
  frame .$FR_cb_vars($FR_cb_path,PATH) -relief flat
  pack .$FR_cb_vars($FR_cb_path,PATH)
    
  set FR_cb_vars($FR_cb_path,LISTBOX) "$FR_cb_vars($FR_cb_path,PATH).lb"
  set FR_cb_vars($FR_cb_path,LB_STATUS) 0
   
  if {[winfo exists .$FR_cb_vars($FR_cb_path,LISTBOX)]} \
  {error "The widget with the path \".$FR_cb_vars($FR_cb_path,LISTBOX)\" already exits. Please choice an other name."}
   
  #---set the default for the -command option
  if {$FR_cb_vars($FR_cb_path,COMMAND_SET) != 1} {set FR_cb_vars($FR_cb_path,COMMAND) ""}
  
  #---safe the value "auto" if set with -entry-width option
  if {$FR_cb_vars($FR_cb_path,ENTRY_WIDTH) eq "auto"} {set FR_cb_vars($FR_cb_path,ENTRY_WIDTH_AUTO) 1} else {set FR_cb_vars($FR_cb_path,ENTRY_WIDTH_AUTO) 0}
  

  #---the image for the button
  if {![info exists FR_cb_image1]} \
  {
   set FR_cb_image1 [image create bitmap -data {#define combobox_width 16 #define combobox_height 16
   static unsigned char combobox_bits[] = {
   0x00, 0x00, 0x80, 0x01, 0x80, 0x01, 0x80, 0x01, 0x80, 0x01, 0x80, 0x01,
   0x80, 0x01, 0x90, 0x09, 0xb0, 0x0d, 0xe0, 0x07, 0xc0, 0x03, 0x80, 0x01,
   0x00, 0x00, 0xf0, 0x0f, 0xf0, 0x0f, 0x00, 0x00};}]
  }
  
  #---we create a new topwindow for the listbox
  toplevel .$FR_cb_vars($FR_cb_path,LISTBOX) -borderwidth 2 -relief groove
   
  listbox .$FR_cb_vars($FR_cb_path,LISTBOX).cb_listbox -activestyle none -yscroll ".$FR_cb_vars($FR_cb_path,LISTBOX).cb_yscrollbar set" -xscroll ".$FR_cb_vars($FR_cb_path,LISTBOX).cb_xscrollbar set" -background white
  
  scrollbar .$FR_cb_vars($FR_cb_path,LISTBOX).cb_yscrollbar -orient vertical -command ".$FR_cb_vars($FR_cb_path,LISTBOX).cb_listbox yview"
  scrollbar .$FR_cb_vars($FR_cb_path,LISTBOX).cb_xscrollbar -orient horizontal -command ".$FR_cb_vars($FR_cb_path,LISTBOX).cb_listbox xview"
   
  grid .$FR_cb_vars($FR_cb_path,LISTBOX).cb_listbox -row 0 -column 0 -sticky nsew
  grid columnconfigure .$FR_cb_vars($FR_cb_path,LISTBOX) 0 -weight 1
  
  wm overrideredirect .$FR_cb_vars($FR_cb_path,LISTBOX) 1
  wm withdraw .$FR_cb_vars($FR_cb_path,LISTBOX)
  
  
  #----------a subproc which execute the -command parameter after a selection in the listbox----------
  proc ::FR_comboBox::combobox_command {widget} \
  {
   variable FR_cb_vars
   
   if {$FR_cb_vars($widget,COMMAND) ne ""} \
   {
    catch [set FR_cb_vars($widget,COMMAND)]
   }
  }
  
  
  #---a subproc which calculate the size of the listbox and pop it up
  proc ::FR_comboBox::combobox_popup {widget} \
  {
   variable FR_cb_vars
   
   set POS_X [winfo rootx .$FR_cb_vars($widget,PATH).cb_entry]
   set POS_Y [expr {[winfo rooty .$FR_cb_vars($widget,PATH).cb_entry]+[winfo height .$FR_cb_vars($widget,PATH).cb_button]}]
   
   set WIDTH_X [expr {[winfo width .$FR_cb_vars($widget,PATH).cb_entry]+[winfo width .$FR_cb_vars($widget,PATH).cb_button]-[.$FR_cb_vars($widget,PATH).cb_button cget -borderwidth]}]
   set HEIGHT_Y [winfo reqheight .$FR_cb_vars($widget,LISTBOX)]
   
   wm geometry .$FR_cb_vars($widget,LISTBOX) ${WIDTH_X}x${HEIGHT_Y}+${POS_X}+${POS_Y}
   wm deiconify .$FR_cb_vars($widget,LISTBOX)
   raise .$FR_cb_vars($widget,LISTBOX)
   
   set FR_cb_vars($widget,LB_STATUS) 1
  }
  
  
  #---a subproc which close the listbox
  proc ::FR_comboBox::combobox_remove {widget} \
  {
   variable FR_cb_vars
   
   wm withdraw .$FR_cb_vars($widget,LISTBOX)
   
   set FR_cb_vars($widget,LB_STATUS) 0
  }
   
  
  #---the action after button pressing
  proc ::FR_comboBox::button_action {widget} \
  {
   variable FR_cb_vars
   
   #---we do it only if button is in the normal state
   if {[$widget cget -state] ne "active"} {return}
   
   set widget [winfo parent $widget]
   set widget [string trimleft $widget .]
   
   if {$FR_cb_vars($widget,LB_STATUS)} \
   {
    ::FR_comboBox::combobox_remove $widget
   } else \
   {
    ::FR_comboBox::combobox_popup $widget
   }
  }
   
  
  #---the action after selection a item in the listbox
  proc ::FR_comboBox::lb_selection {widget} \
  {
   set index [$widget curselection] 
   set selected [$widget get $index]
   
   set widget [winfo parent [winfo parent $widget]]
   set widget [string trimleft $widget .]
   
   #---we remove the listbox, put the selection in the entry widget and execute the -command argument
   ::FR_comboBox::combobox_remove $widget
   
   ::FR_comboBox::cb_insert_entry $widget $selected
   
   ::FR_comboBox::combobox_command $widget
  }
  
  
  if {$FR_cb_vars($FR_cb_path,ENTRY_WIDTH) eq "auto"} \
  {
   eval entry .$FR_cb_vars($FR_cb_path,PATH).cb_entry -disabledforeground black -disabledbackground white -width 2 -state disable -textvariable [set FR_cb_vars($FR_cb_path,VARIABLE)]
  } else \
  {
   eval entry .$FR_cb_vars($FR_cb_path,PATH).cb_entry -disabledforeground black -disabledbackground white -width $FR_cb_vars($FR_cb_path,ENTRY_WIDTH) -state disable -textvariable [set FR_cb_vars($FR_cb_path,VARIABLE)]
  }
  
  pack .$FR_cb_vars($FR_cb_path,PATH).cb_entry -side left -fill x -expand yes
  
  button .$FR_cb_vars($FR_cb_path,PATH).cb_button -image $FR_cb_image1
  
  pack .$FR_cb_vars($FR_cb_path,PATH).cb_button -side right
  
  bind .$FR_cb_vars($FR_cb_path,PATH).cb_button <ButtonPress-1> {::FR_comboBox::button_action %W}  
  bind .$FR_cb_vars($FR_cb_path,LISTBOX).cb_listbox <<ListboxSelect>> {::FR_comboBox::lb_selection %W}
 
  if {!$FR_cb_vars($FR_cb_path,PARENT_SET)} {set FR_cb_vars($FR_cb_path,PARENT) .}
  if {[winfo exists $FR_cb_vars($FR_cb_path,PARENT)]} \
  {
   bindtags $FR_cb_vars($FR_cb_path,PARENT) [linsert [bindtags $FR_cb_vars($FR_cb_path,PARENT)] 0 configure_refresh]
  } else {error "I didn't find the \"-parent\" window: $FR_cb_vars($FR_cb_path,PARENT)"}
  bind configure_refresh <Configure> {::FR_comboBox::lb_remove}
  
  #---if you overtake some contents we fill it immediately
  if {$FR_cb_vars($FR_cb_path,LB_CONTENT_SET)} {::FR_comboBox::fill_lb}
 }

 
 
 #---if the combox not yet exists -> we create one
 if {$FR_cb_vars($FR_cb_path,EXIST) == 0} \
 {
  ::FR_comboBox::create
  
  return $FR_cb_vars($FR_cb_path,PATH)
 }
 
 
 #---start the filling ("-listbox_content") of the combobox
 if {$FR_cb_vars($FR_cb_path,LB_CONTENT_SET)} {::FR_comboBox::fill_lb}
 
 
 #---start the configuration ("-conf_widget" and "-conf_option") of the combobox 
 if {$FR_cb_vars($FR_cb_path,CONF_WIDGET_SET) && $FR_cb_vars($FR_cb_path,CONF_OPTION_SET)} \
 {::FR_comboBox::FR_comboBox_config}
 
 
 #---start the "-state" option as a subversion of configuration
 if {$FR_cb_vars($FR_cb_path,STATE_SET)} \
 {
  set FR_cb_vars($FR_cb_path,CONF_WIDGET) "button"
  set FR_cb_vars($FR_cb_path,CONF_OPTION) "-state $FR_cb_vars($FR_cb_path,STATE)"
  
  ::FR_comboBox::FR_comboBox_config
  
  set FR_cb_vars($FR_cb_path,CONF_WIDGET) "entry"
  
  if {$FR_cb_vars($FR_cb_path,STATE) eq "normal"} \
  {
   set FR_cb_vars($FR_cb_path,CONF_OPTION) "-disabledforeground black"
  } else \
  {
   set FR_cb_vars($FR_cb_path,CONF_OPTION) "-disabledforeground grey"
  }
  
  ::FR_comboBox::FR_comboBox_config 
 }
 
 
 #---start the "-read" option
 if {$FR_cb_vars($FR_cb_path,READ_SET)} \
 {::FR_comboBox::FR_comboBox_read $FR_cb_vars($FR_cb_path,PATH)}
  
}


#---if . configures we remove all open listboxes
proc ::FR_comboBox::lb_remove {} \
{
 variable FR_cb_vars
 
 set ALL_CB ""; set ALL_CB_NEW ""
 
 #---first we search all comboboxes
 if {[array exists FR_cb_vars]} \
 {
  set ALL_CB [array names FR_cb_vars "*,LB_STATUS"]
 
  foreach ARG $ALL_CB \
  {
   set ARG [string trimright $ARG ",LB_STATUS"]
   lappend ALL_CB_NEW $ARG
  }
  
  set ALL_CB $ALL_CB_NEW
 }
 
 #---now we close all open comboboxes
 if {$ALL_CB ne ""} \
 {
  foreach ARG $ALL_CB \
  {
   if {$FR_cb_vars($ARG,LB_STATUS)} \
   {
    wm withdraw .$FR_cb_vars($ARG,LISTBOX)
   
    set FR_cb_vars($ARG,LB_STATUS) 0
   }
  }
 } else {puts "No comboboxes found"}

}
