#---this create a canvas with a xy-diagram (x-axis can be linear or logarithmical)
#
# usage
#
# ::FR_diagram::create
# --------------------
#
# create a 2D diagram; can also draw datas and show a diagram title in one step
# -window          {the path to the wished canvas}
# -x_axis_line_cnt {count of vertical grid lines | default: 5}
# -y_axis_line_cnt {count of horizontal grid lines | default: 11}
# -x_axis_min      {minimum value of the x-axis | default: 0}
# -x_axis_max      {maximum value of the x-axis | default: 1}
# -y_axis_min      {minimum value of the y-axis | default: -1}
# -y_axis_max      {maximum value of the y-axis | default: 1}
# -data            {list with xy-values in the from x1 y1 x2 y2...}
# -x_axis_log      {set 1 if you want to have a logarithmical x-axis | default: 0}
# -diagram_title   {diagram title | default: ""}
#
# ::FR_diagram::draw_data
# -----------------------
# 
# erase the old curve and draw new datas
# ::FR_diagram::draw_data {Data {X_Axis_Log 0}}
#
# ::FR_diagram::diagram_title
# ---------------------------
# erase the old and set a new diagram title
# ::FR_diagram::diagram_title {string}
#
# ::FR_diagram::zoom_out
# ---------------------------
# go back to the last zoom (view_all=0) or 
# draw the whole curve (view_all=1)
# ::FR_diagram::zoom_out {view_all}
#

package provide FR_diagram 0.1

namespace eval ::FR_diagram \
{
 namespace export create diagram_title draw_data zoom_out
}


#---the start point
proc ::FR_diagram::create {args} \
{ 
 variable data; variable can; variable VAR
 
 # we set the default values
 set can ""
 set VAR(x_axis_line_cnt) 5
 set VAR(y_axis_line_cnt) 11
 set VAR(x_axis_log) 0
 set VAR(x_data_min) 0
 set VAR(x_data_max) 1
 set VAR(y_data_min) -1
 set VAR(y_data_max) 1
 set VAR(diagram_title) ""
 set data ""
 set VAR(data_range) ""
 set VAR(redraw_request) ""
 
 for {set i 0} {$i < [llength $args]} {incr i 2} \
 {
  set arg [lindex $args $i]

  switch -glob -- $arg \
  {-window          {set can [lindex $args [expr {$i+1}]]} \
   -x_axis_line_cnt {set VAR(x_axis_line_cnt) [lindex $args [expr {$i+1}]]} \
   -y_axis_line_cnt {set VAR(y_axis_line_cnt) [lindex $args [expr {$i+1}]]} \
   -x_axis_min      {set VAR(x_data_min) [lindex $args [expr {$i+1}]]} \
   -x_axis_max      {set VAR(x_data_max) [lindex $args [expr {$i+1}]]} \
   -y_axis_min      {set VAR(y_data_min) [lindex $args [expr {$i+1}]]} \
   -y_axis_max      {set VAR(y_data_max) [lindex $args [expr {$i+1}]]} \
   -x_axis_log      {set VAR(x_axis_log) [lindex $args [expr {$i+1}]]} \
   -diagram_title   {set VAR(diagram_title) [lindex $args [expr {$i+1}]]} \
   -data            {set data [lindex $args [expr {$i+1}]]} \
   --               {break} \
   *                {error "unknown option: $arg"}
   } 
  }
 
 # we need a path for the canvas widget
 if {$can eq ""} {error "You must set a window name for your diagram with \"-window\""}
 set can [string trimleft $can .]
 
 # we need at least 2 lines: for the minimum and the maximum value
 if {$VAR(x_axis_line_cnt) < 2} {set VAR(x_axis_line_cnt) 2}
 if {$VAR(y_axis_line_cnt) < 2} {set VAR(y_axis_line_cnt) 2}
 
 ::FR_diagram::init
 ::FR_diagram::look
 
 ::FR_diagram::draw_y_axis $VAR(y_data_min) $VAR(y_data_max)
 
 if {$VAR(x_axis_log)} \
 {
  ::FR_diagram::draw_log_x_axis $VAR(x_data_min) $VAR(x_data_max)
 } else \
 {
  ::FR_diagram::draw_x_axis $VAR(x_data_min) $VAR(x_data_max)
 }
 
 # we draw the curve
 set data_length [llength $data]
 if {$data_length > 3} \
 { 
  set VAR(last_element) [expr {$data_length - 1}]
  ::FR_diagram::draw_curve [list 0 $VAR(last_element)]
  # we preset the zoom history with the whole plot
  set VAR(zoom_history) [list 0 $VAR(last_element)] 
 }
 
 if {$VAR(diagram_title) ne ""} \
 {
  ::FR_diagram::diagram_title
 }
}


#---we create a canvas and set the bindings
proc ::FR_diagram::init {} \
{
 variable can
 
 if {[winfo exists .$can]} {destroy .$can}
 
 canvas .$can -background white
 pack   .$can -fill both -expand yes
 
 # the menu for the zoom operations
 menu .$can.menu -tearoff 0
 .$can.menu add command -label [msgcat::mc "Zoom out"] -command [list ::FR_diagram::zoom_out 0]
 .$can.menu add separator
 .$can.menu add command -label [msgcat::mc "View all"] -command [list ::FR_diagram::zoom_out 1]
 
 bind .$can <ButtonPress-1> "::FR_diagram::start_zoom_box %x %y"
 bind .$can <Motion> "::FR_diagram::draw_zoom_box %x %y"
 bind .$can <ButtonRelease-1> "::FR_diagram::end_zoom_box %x %y"
 bind .$can <Button-3> [list tk_popup .$can.menu %X %Y]
 
 ::FR_diagram::getsize
 
 # we bind the parent of canvas to its Configure command
 # to redraw the canvas
 set canvas_parent [winfo parent .$can]
 if {$canvas_parent eq ""} {set canvas_parent .}
 
 bind $canvas_parent <Configure> "::FR_diagram::redraw_request"
}


#---we ask for the whole size of the canvas (we need this for the redraw function)
proc ::FR_diagram::get_canvas_size {} \
{
 variable VAR; variable can
 
 set VAR(canvas_x_size) [winfo width .$can]
 set VAR(canvas_y_size) [winfo height .$can]
}


#---returns the internal x-size of the canvas
proc ::FR_diagram::getxsize {} \
{
 variable VAR; variable can
 
 set VAR(cmx) [ expr {[winfo width .$can]-2*[.$can cget -border]-2*[.$can cget -highlightthickness]} ]
 
 if {$VAR(cmx) <= 1} {set VAR(cmx) [.$can cget -width]}
}


#---returns the internal y-size of the canvas
proc ::FR_diagram::getysize {} \
{
 variable VAR; variable can
 
 set VAR(cmy) [ expr {[winfo height .$can]-2*[.$can cget -border]-2*[.$can cget -highlightthickness]} ]
 
 if {$VAR(cmy) <= 1} {set VAR(cmy) [.$can cget -height]}
}


#---we ask for the size of the canvas (inside, the useable painting area)
proc ::FR_diagram::getsize {} \
{
 update
 
 ::FR_diagram::get_canvas_size
 
 ::FR_diagram::getxsize
 ::FR_diagram::getysize
}


#---precalculation of the diagram size
proc ::FR_diagram::look {} \
{
 variable can; variable VAR

 #---the spaces around the diagram
 set bottom 0.1
 set left   0.1
 set right  0.05
 set top    0.1
 
 set VAR(y_axis_range) [expr {$VAR(cmy) * (1 - $bottom - $top)}]
 set VAR(delta_y_axis_lines) [expr {$VAR(y_axis_range) / ($VAR(y_axis_line_cnt) - 1)}]
 
 set VAR(y_axis_text_xpos) [expr {$VAR(cmx) * $left * 0.75}]
 set VAR(y_axis_line_xpos) [expr {$VAR(cmx) * (0.01 + $left)}]
 
 set VAR(x_axis_range) [expr {$VAR(cmx) * (1 - $left - $right)}]
 set VAR(delta_x_axis_lines) [expr {$VAR(x_axis_range) / ($VAR(x_axis_line_cnt) - 1)}]
 
 set VAR(x_axis_text_ypos) [expr {$VAR(cmy) * (1 - $bottom * 0.5)}]
 set VAR(x_axis_line_ypos) [expr {$VAR(cmy) * (1 - (0.01 + $bottom))}]
 
 set VAR(y_axis_min) [expr {$VAR(cmy) * $top}]
 set VAR(y_axis_max) [expr {$VAR(cmy) * (1 - $bottom)}]
 
 set VAR(x_axis_min) [expr {$VAR(cmx) * $left}]
 set VAR(x_axis_max) [expr {$VAR(cmx) * (1 - $right)}]
 
 set VAR(diagram_title_ypos) [expr {$VAR(cmy) * $top * 0.5}]
 set VAR(diagram_title_xpos) [expr {($VAR(x_axis_range) / 2) + $VAR(x_axis_min)}]
}


#---draws the linear y-axis and the horizontal grid
proc ::FR_diagram::draw_y_axis {y_min y_max {lettering_only 0}} \
{
 variable can; variable VAR
 
 set VAR(y_data_min) $y_min
 set VAR(y_data_max) $y_max
 set VAR(y_data_range) [expr {$y_max - $y_min}]
 
 
 # we create the min and max line
 foreach {y i} [list $VAR(y_axis_min) $VAR(y_data_max) $VAR(y_axis_max) $VAR(y_data_min)] \
 {
  eval .$can create text $VAR(y_axis_text_xpos) $y -text [format "%.6f" $i] -fill black -anchor e -tag y_axis_text
  if {!$lettering_only} \
  {
   .$can create line $VAR(x_axis_min) $y $VAR(x_axis_max) $y -fill black -width 1.000000 -capstyle round -tag y_axis_grid
  }
 }
 
 # if only two lines are wished (min and max) we have already done our job
 if {$VAR(y_axis_line_cnt) == 2} {return}
 
 # the rise of the value of lettering
 set tmp [expr {1.0 * $VAR(y_data_range) / ($VAR(y_axis_line_cnt) - 1)}]
 
 for {set i 1} {$i < ($VAR(y_axis_line_cnt)-1)} {incr i}\
 {
  # the y position for the lettering and the lines
  set y [expr {$i * $VAR(delta_y_axis_lines) + $VAR(y_axis_min)}]
  
  # the value of the lettering
  set value [expr {-$tmp * $i + $VAR(y_data_max)}]
  
  eval .$can create text $VAR(y_axis_text_xpos) $y -text [format "%.6f" $value] -fill black -anchor e -tag y_axis_text
  
  if {!$lettering_only} \
  {
   .$can create line $VAR(x_axis_min) $y $VAR(x_axis_max) $y -fill gray -width 1.000000 -capstyle round -tag y_axis_grid
   .$can create line $VAR(x_axis_min) $y $VAR(y_axis_line_xpos) $y -fill black -width 1.000000 -capstyle round -tag y_axis_line
  }
 }
}


#---draws the linear x-axis and the vertical grid
proc ::FR_diagram::draw_x_axis {x_min x_max {lettering_only 0}} \
{
 variable can; variable VAR
 
 set VAR(x_data_min) $x_min
 set VAR(x_data_max) $x_max
 set VAR(x_data_range) [expr {$x_max - $x_min}]

 # we create the min and max line
 foreach {x i} [list $VAR(x_axis_min) $x_min $VAR(x_axis_max) $x_max] \
 {
  eval .$can create text $x $VAR(x_axis_text_ypos) -text [format "%.6f" $i] -fill black -anchor center -tag x_axis_text
  if {!$lettering_only} \
  {
   .$can create line $x $VAR(y_axis_min) $x $VAR(y_axis_max) -fill black -width 1.000000 -capstyle round -tag x_axis_grid
  }
 }

 # if only two lines are wished (min and max) we have already done our job 
 if {$VAR(x_axis_line_cnt) == 2} {return}
 
 # the rise of the value of lettering
 set tmp [expr {1.0 * $VAR(x_data_range) / ($VAR(x_axis_line_cnt) - 1)}]
 
 for {set i 1} {$i < ($VAR(x_axis_line_cnt)-1)} {incr i}\
 {
  # the x position for the lettering and the lines
  set x [expr {$i * $VAR(delta_x_axis_lines) + $VAR(x_axis_min)}]
  
  # the value of the lettering
  set value [expr {$tmp * $i + $x_min}]
  
  eval .$can create text $x $VAR(x_axis_text_ypos) -text [format "%.6f" $value] -fill black -anchor center -tag x_axis_text
  
  if {!$lettering_only} \
  {
   .$can create line $x $VAR(y_axis_min) $x $VAR(y_axis_max) -fill gray -width 1.000000 -capstyle round -tag x_axis_grid
   .$can create line $x $VAR(y_axis_max) $x $VAR(x_axis_line_ypos) -fill black -width 1.000000 -capstyle round -tag x_axis_line
  }
 }
}


#---draws the logarithmical x-axis and the vertical grid
proc ::FR_diagram::draw_log_x_axis {x_min x_max {lettering_only 0}} \
{
 variable can; variable VAR
 
 # Falk
 # the zoom proc overtakes logarthimical values
# if {$zoom} \
# {
#  set x_min [expr {pow(10,$x_min)}]
#  set x_max [expr {pow(10,$x_max)}]
# }
 
 
 # this looks for the segment in which is value
 # and gives back the start value of this segment
 proc segment {value} \
 {
  if {$value < 10}      {return 1}
  if {$value < 100}     {return 10}
  if {$value < 1000}    {return 100}
  if {$value < 10000}   {return 1000}
  if {$value < 100000}  {return 10000}
 }
 
 
 # we set x_data_min to the next valid element in the segment
 set x_min_seg_start [segment $x_min]
 set x_min_seg_end [expr {$x_min_seg_start * 10}]
 
 set tmp 0
 
 for {set i $x_min_seg_start} {$i <= $x_min_seg_end} {incr i $x_min_seg_start} \
 {
  if {$i == $x_min} {break}
  if {$i > $x_min} {set tmp $x_min_seg_start; break}
 }
 
 # if i > x_min so we are over the limit
 # we go one step back
 set i [expr {$i - $tmp}]
 
 set x_data_min_lin $i
 set VAR(x_data_min) [expr {log10($i)}]
 
 # we set x_data_max to the next valid element in the segment
 set x_max_seg_start [segment $x_max]
 set x_max_seg_end [expr {$x_max_seg_start * 10}]
 
 for {set i $x_max_seg_start} {$i <= $x_max_seg_end} {incr i $x_max_seg_start} \
 {
  if {$i >= $x_max} {break}
 }
 
 if {$i == $x_data_min_lin} {set i [expr{$i + $x_max_seg_start}]}
 
 set x_data_max_lin $i
 set VAR(x_data_max) [expr {log10($i)}]
 
 set VAR(x_data_range) [expr {$VAR(x_data_max) - $VAR(x_data_min)}]
 
 # if value > 1000 we replace this through k(ilo)
 proc transfer_label {value} \
 {
  set tmp [expr {$value / 1000}]
  
  if {$tmp >= 1} {return "${tmp}k"} else {return $value}
 }
 
 # we create the min and max line
 foreach {x i} [list $VAR(x_axis_min) $x_data_min_lin $VAR(x_axis_max) $x_data_max_lin] \
 {
  set label_text [transfer_label $i]
  eval .$can create text $x $VAR(x_axis_text_ypos) -text $label_text -fill black -anchor center -tag x_axis_text
  
  if {!$lettering_only} \
  {
   .$can create line $x $VAR(y_axis_min) $x $VAR(y_axis_max) -fill black -width 1.000000 -capstyle round -tag x_axis_grid
  }
 }

 set run 1
 set x_axis_values ""
 
 for {set i $x_min_seg_start} {($i <= $x_max_seg_start) && $run} {set i [expr {$i * 10}]} \
 {
  set end [expr {$i * 10}]
  for {set j $i} {$j < $end} {incr j $i} \
  {
   if {$j <= $x_data_min_lin} {continue}
   if {$j >= $x_data_max_lin} {set run 0; break}
   
   lappend x_axis_values $j
  }
 }

 set delta_x_axis [expr {$VAR(x_axis_range) / ($VAR(x_data_max) - $VAR(x_data_min))}]

 foreach i $x_axis_values \
 {
  set x [expr {$delta_x_axis * (log10($i) - $VAR(x_data_min)) + $VAR(x_axis_min)}]
  
  set run 1
  
  # we don't write the 7 and 9 label (to save space on the axis) 
  if {![expr {$i % 7}] || ![expr {$i % 9}]} {set run 0}
  
  if {$run} \
  {
   eval .$can create text $x $VAR(x_axis_text_ypos) -text [transfer_label $i] -fill black -anchor center -tag x_axis_text
  }
  
  if {!$lettering_only} \
  {
   .$can create line $x $VAR(y_axis_min) $x $VAR(y_axis_max) -fill gray -width 1.000000 -capstyle round -tag x_axis_grid
   .$can create line $x $VAR(y_axis_max) $x $VAR(x_axis_line_ypos) -fill black -width 1.000000 -capstyle round -tag x_axis_line
  }
 }
}


#---this plots the data
proc ::FR_diagram::draw_curve {data_range} \
{
 variable can; variable VAR
 variable data;
 
 # if no data range is specified we havn't anything to do...
 if {[llength $data_range] !=  2} {return 0}
 
 set VAR(data_range) $data_range
 
 set delta_x_data [expr {$VAR(x_axis_range) / $VAR(x_data_range)}]
 set delta_y_data [expr {$VAR(y_axis_range) / $VAR(y_data_range)}]
 
 # with i we make a delay of one values pairs to have start and end value
 set i 0
 
 foreach {x y} [lrange $data [lindex $VAR(data_range) 0] [lindex $VAR(data_range) 1]] \
 {
  if {$VAR(x_axis_log)} \
  {
   set x [expr {log10($x)}]
  }
  
  set x_end $x
  set y_end $y
  
  # in the first loop we don't come in this body, so we can define x_start
  # and y_start at the end of this foreach body
  if {$i} \
  {
   #we transfere the xy-data to xy-canvas
   set x_start_can [expr {$delta_x_data * ($x_start - $VAR(x_data_min)) + $VAR(x_axis_min)}]
   set x_end_can [expr {$delta_x_data * ($x_end - $VAR(x_data_min)) + $VAR(x_axis_min)}]
   
   set y_start_can [expr {$delta_y_data * ($VAR(y_data_max) - $y_start) + $VAR(y_axis_min)}]
   set y_end_can [expr {$delta_y_data * ($VAR(y_data_max) - $y_end) + $VAR(y_axis_min)}]
   
   #we check if the xy-canvas are outside of the diagram
   set y_start_outside 0; set y_end_outside 0
   if {($y_start_can > $VAR(y_axis_max)) || ($y_start_can < $VAR(y_axis_min))} {set y_start_outside 1}
   if {($y_end_can > $VAR(y_axis_max)) || ($y_end_can < $VAR(y_axis_min))} {set y_end_outside 1}
   
   set draw_line 0
   
   # if the start_values is inside and the end_value is outside
   if {(!$y_start_outside && $y_end_outside)} \
   {
    # we save the y_end_can value for the calculation of x_end_can
    set y_end_can_real $y_end_can
    
    if {$y_end_can > $VAR(y_axis_max)} \
    {
     set y_end_can $VAR(y_axis_max)
    } else \
    {
     # if y_end_can < $y_axis_min because we checked before for to be outside
     set y_end_can $VAR(y_axis_min)
    }
    
    # we set the x_end values to the place where the line goes out (y_axis_max or y_axis_min)
    set x_end_can [expr {($y_end_can - $y_start_can) * ($x_end_can - $x_start_can) / ($y_end_can_real - $y_start_can) + $x_start_can}]
    
    
    set draw_line 1
   }
   
   # if the start_values is outside and the end_value is inside
   if {($y_start_outside && !$y_end_outside)} \
   {
    # we save the y_start_can value for the calculation of x_start_can
    set y_start_can_real $y_start_can
    
    if {$y_start_can > $VAR(y_axis_max)} \
    {
     set y_start_can $VAR(y_axis_max)
    } else \
    {
     #y_start_can must be < $y_axis_min because we checked before for to be outside
     set y_start_can $VAR(y_axis_min)
    }
    
    # we set the x_start values to place where the lines comes in (y_axis_max or y_axis_min)
    set x_start_can [expr {($y_start_can - $y_start_can_real) * ($x_end_can - $x_start_can) / ($y_end_can - $y_start_can_real) + $x_start_can}]
    
    set draw_line 1
   }
       
   #if both values are inside in the diagram (the normal case)
   if {(!$y_start_outside && !$y_end_outside) || $draw_line} \
   {
    .$can create line $x_start_can $y_start_can $x_end_can $y_end_can -fill blue -width 1.000000 -capstyle round -tag data_curve
   }
  }
  
  set x_start $x_end
  set y_start $y_end
  set i 1 
 }
}


#---this add a title to the diagram
proc ::FR_diagram::diagram_title {{title ""}} \
{
 variable can; variable VAR
 
 ::FR_diagram::delete diagram_title
 
 if {$title ne ""} {set VAR(diagram_title) $title}
 
 set diagram_title [string range $VAR(diagram_title) 0 99]
 
 .$can create text $VAR(diagram_title_xpos) $VAR(diagram_title_ypos) -text $diagram_title -fill black -anchor center -tag diagram_title
}


#---delete tag-list in can
proc ::FR_diagram::delete {{tag ""}} \
{
 variable can
 
 if {$tag eq ""} {return}

 foreach i $tag {.$can delete $i}
}


#---we check if x_max > x_min and y_max > y_min
proc ::FR_diagram::check_limits {x_min x_max y_min y_max} \
{ 
 set limits_error 0
 
 if {$x_max <= $x_min} \
 {
  set limits_error 1
  return $limits_error
 }
 
 if {$y_max < $y_min} \
 {
  set limits_error 1
  return $limits_error
 }
 
 if {$y_max == $y_min} \
 {
  set limits_error 2
  return $limits_error
 }
 
 return $limits_error
}


#---saves the start of the rectangle zoom box
proc ::FR_diagram::start_zoom_box {x y} \
{
 variable VAR 
 
 set VAR(start_zoom_box) 1
 set VAR(x_start_zoom_box) $x
 set VAR(y_start_zoom_box) $y
}


#---saves the end of the rectangle zoom box
proc ::FR_diagram::end_zoom_box {x y} \
{
 variable VAR
 
 set VAR(start_zoom_box) 0
 set VAR(x_end_zoom_box) $x
 set VAR(y_end_zoom_box) $y
 
 ::FR_diagram::delete zoom_box

 ::FR_diagram::check_zoom_box
 ::FR_diagram::zoom_calc
}


#---draws the rectangle zoom box
proc ::FR_diagram::draw_zoom_box {x y} \
{
 variable can; variable VAR
 
 if {![info exists VAR(start_zoom_box)] || !$VAR(start_zoom_box)} {return}
 
 ::FR_diagram::delete zoom_box 
 
 .$can create rectangle $VAR(x_start_zoom_box) $VAR(y_start_zoom_box) $x $y -outline black -tags zoom_box 
}


#---checks and corrects if the zoom box is outside of the diagram
#---we also check if the box drawn from right to left 
proc ::FR_diagram::check_zoom_box {} \
{
 variable VAR
 
 # first we limit zoom box coordinates to the min/max axis values 
 if {$VAR(x_start_zoom_box) < $VAR(x_axis_min)} {set VAR(x_start_zoom_box) $VAR(x_axis_min)}
 if {$VAR(x_end_zoom_box) > $VAR(x_axis_max)} {set VAR(x_end_zoom_box) $VAR(x_axis_max)}
 
 if {$VAR(y_start_zoom_box) < $VAR(y_axis_min)} {set VAR(y_start_zoom_box) $VAR(y_axis_min)}
 if {$VAR(y_end_zoom_box) > $VAR(y_axis_max)} {set VAR(y_end_zoom_box) $VAR(y_axis_max)}
 
 # we check if the box drawn from right to left
 if {$VAR(x_start_zoom_box) > $VAR(x_end_zoom_box)} \
 {
  set tmp $VAR(x_start_zoom_box)
  set VAR(x_start_zoom_box) $VAR(x_end_zoom_box)
  set VAR(x_end_zoom_box) $tmp
  
  set tmp $VAR(y_start_zoom_box)
  set VAR(y_start_zoom_box) $VAR(y_end_zoom_box)
  set VAR(y_end_zoom_box) $tmp
  
  unset tmp
 }
}


#---we calculate back from xy-canvas to xy-data
proc ::FR_diagram::zoom_calc {} \
{
 variable VAR; variable data

 set tmp [expr {$VAR(x_data_range) / $VAR(x_axis_range)}]
 
 set x_min [expr {($VAR(x_start_zoom_box) - $VAR(x_axis_min)) * $tmp + $VAR(x_data_min)}]
 set x_max [expr {($VAR(x_end_zoom_box) - $VAR(x_axis_min)) * $tmp + $VAR(x_data_min)}]
 
 set tmp [expr {$VAR(y_data_range) / $VAR(y_axis_range)}] 
 
 set y_max [expr {$VAR(y_data_max)-($VAR(y_start_zoom_box) - $VAR(y_axis_min)) * $tmp}]
 set y_min [expr {$VAR(y_data_max)-($VAR(y_end_zoom_box) - $VAR(y_axis_min)) * $tmp}]
 
 # if x_max <= x_min or y_max <= y_min we can't further zoom
 set limits_error [::FR_diagram::check_limits $x_min $x_max $y_min $y_max]
 if {$limits_error} {return}
 
 # we select out the needed x-values (from data) and start the drawing
 if {[llength $VAR(data_range)] !=  2} {return 0}
 
 set start_index [lindex $VAR(data_range) 0]
 set end_index [lindex $VAR(data_range) 1]
 
 #we are looking for x > minimum
 for {set i $start_index} {$i < $end_index} {incr i 2} \
 {
  set tmp [lindex $data $i]
  
  if {$VAR(x_axis_log)} {set tmp [expr {log10($tmp)}]}
  
  if {$tmp >= $x_min} {break}
 }
 
 #we are looking for x < maximum
 for {set j $i} {$j < $end_index} {incr j 2} \
 {
  set tmp [lindex $data $j]
  
  if {$VAR(x_axis_log)} {set tmp [expr {log10($tmp)}]}
  
  if {$tmp > $x_max} {break}
 }
 
 # with the break command we are one element-pair to far
 # only -1 because we need also the y-value at x_data_max
 incr j -1
 
 # we need at least three x (and three y) values for further zooming
 if {[expr {abs($i - $j)}] < 5} {return}
 
 # we save the indeces i and j and append it to the zoom history
 set VAR(data_range) [list $i $j]
 lappend VAR(zoom_history) $i $j
 
 ::FR_diagram::zoom_draw $y_min $y_max
}


proc ::FR_diagram::zoom_draw {y_min y_max} \
{
 variable VAR; variable data
 
 #x_min is the first and x_max the last point of data
 set start_index [lindex $VAR(data_range) 0]
 set end_index [lindex $VAR(data_range) 1]
 
 set x_min [lindex $data $start_index]
 set x_max [lindex $data [expr {$end_index - 1}]]
 
 # now we draw all
 if {$VAR(x_axis_log)} \
 {
  set tmp [list data_curve x_axis_text x_axis_grid x_axis_line y_axis_text]
 } else \
 {
  set tmp [list data_curve x_axis_text y_axis_text]
 }
 
 ::FR_diagram::delete $tmp
 
 ::FR_diagram::draw_y_axis $y_min $y_max 1
 
 if {$VAR(x_axis_log)} \
 {
  ::FR_diagram::draw_log_x_axis $x_min $x_max 1
 } else \
 {
  ::FR_diagram::draw_x_axis $x_min $x_max 1
 }
 
 ::FR_diagram::draw_curve $VAR(data_range)
}

#---the zoom out function
proc ::FR_diagram::zoom_out {view_all} \
{
 variable data; variable VAR
 
 if {$view_all == 0} {
  # this is the normal zoom out function
  
  # we check if the zoom_history has already reached the full picture
  if {[llength $VAR(zoom_history)] < 3} {return}
 
  # if the list length of data smaller as the wished index (maximum)
  # we abort the function, (end-2) because we want the last plot (and not the current plot)
  if {[llength $data] < [lindex $VAR(zoom_history) end-2]} {return}
 
  set VAR(data_range) [list [lindex $VAR(zoom_history) end-3] [lindex $VAR(zoom_history) end-2]]
 } else {
  # this is the view all function 
  set VAR(data_range) [list 0 $VAR(last_element)]
 }
 
 # we are searching the highest and the lowest y value
 set start_index [lindex $VAR(data_range) 0]
 # we are interested in the y-values
 incr start_index
 set end_index [lindex $VAR(data_range) 1]
 
 set y_min [lindex $data $start_index]
 set y_max $y_min
 
 for {set i $start_index} {$i <= $end_index} {incr i 2} \
 {
  set tmp [lindex $data $i]
  
  if {$tmp > $y_max} \
  {
   set y_max $tmp
   continue
  }
   
  if {$tmp < $y_min} \
  {
   set y_min $tmp
  }
 } 
   
 ::FR_diagram::zoom_draw $y_min $y_max
 
 if {$view_all == 0} {
  # now we drew one zoom out so can delete the last zoom
  set VAR(zoom_history) [lrange $VAR(zoom_history) 0 end-2]
 } else {
  # we drew all datas (view_all==1) so we can delete the whole zoom history
  set VAR(zoom_history) [list 0 $VAR(last_element)]
 }
}


#---we will wait until the last configure event
proc ::FR_diagram::redraw_request {} \
{
 variable VAR
 
 if {$VAR(redraw_request) eq ""} \
 {
  set VAR(redraw_request) [after idle ::FR_diagram::redraw]
 }
}

#---the redraw function in the case of a resizing of the canvas widget
proc ::FR_diagram::redraw {} \
{
 variable VAR

 set VAR(canvas_x_size_old) $VAR(canvas_x_size)
 set VAR(canvas_y_size_old) $VAR(canvas_y_size)
 
 update
 
 ::FR_diagram::get_canvas_size
 
 if {($VAR(canvas_x_size_old) != $VAR(canvas_x_size)) || ($VAR(canvas_y_size_old) != $VAR(canvas_y_size))} \
 {
  ::FR_diagram::redraw_sub
 }
 
 set VAR(redraw_request) ""
}

proc ::FR_diagram::redraw_sub {} \
{
 variable can; variable VAR
 
 ::FR_diagram::init
 ::FR_diagram::look
 ::FR_diagram::draw_y_axis $VAR(y_data_min) $VAR(y_data_max)
 
 if {$VAR(x_axis_log)} \
 {
  ::FR_diagram::draw_log_x_axis $VAR(x_data_min) $VAR(x_data_max)
 } else \
 {
  ::FR_diagram::draw_x_axis $VAR(x_data_min) $VAR(x_data_max)
 }
 
 if {[llength $VAR(data_range)] ==  2} \
 {
  ::FR_diagram::draw_curve $VAR(data_range)
 }
 
 if {$VAR(diagram_title) ne ""} \
 {
  ::FR_diagram::diagram_title
 }
}


#---we analyse and draw the data
proc ::FR_diagram::draw_data {Data {X_Axis_Log 0}} \
{
 variable data; variable VAR
 
 # we check: is the data list valid
 set data_length [llength $Data]
 if {$data_length < 4} {return 0}
 
 set data $Data
 set VAR(last_element) [expr {$data_length - 1}]
 set VAR(data_range) [list 0 $VAR(last_element)]
 
 # we set x_min and x_max to the first and last x element in the data list
 set x_min [lindex $data 0]
 set x_max [lindex $data end-1]
 
 # we are searching the highest and the lowest y value
 set y_min [lindex $data 1]
 set y_max $y_min
 
 for {set i 1} {$i < $data_length} {incr i 2} \
 {
  set tmp [lindex $data $i]
  
  if {$tmp > $y_max} \
  {
   set y_max $tmp
   continue
  }
   
  if {$tmp < $y_min} \
  {
   set y_min $tmp
  }
 }
 
 # if x_max <= x_min or y_max < y_min we can't draw the curve
 set limits_error [::FR_diagram::check_limits $x_min $x_max $y_min $y_max]
 if {$limits_error == 1} {return}
 if {$limits_error == 2} \
 {
  set y_min [expr {$y_min - 1}]
  set y_max [expr {$y_max + 1}]
 }
 
 # now we can make the values to official values
 set VAR(x_data_min) $x_min
 set VAR(x_data_max) $x_max
 set VAR(y_data_min) $y_min
 set VAR(y_data_max) $y_max 
 
 # we reset the history of the last zooms
 set VAR(zoom_history) [list 0 $VAR(last_element)]

 
 # we delete the old things in the diagram
 if {$X_Axis_Log != $VAR(x_axis_log)} \
 { 
  set tmp [list data_curve x_axis_text x_axis_grid x_axis_line y_axis_text]
 } else \
 {
  set tmp [list data_curve x_axis_text y_axis_text]
 }
 
 ::FR_diagram::delete $tmp
 
 # we draw the y axis new 
 ::FR_diagram::draw_y_axis $VAR(y_data_min) $VAR(y_data_max) 1
 
 # we draw the x axis new
 if {$X_Axis_Log} \
 {
  set VAR(x_axis_log) 1
  ::FR_diagram::draw_log_x_axis $VAR(x_data_min) $VAR(x_data_max) 0
 } else \
 {
  set VAR(x_axis_log) 0
  ::FR_diagram::draw_x_axis $VAR(x_data_min) $VAR(x_data_max) 0
 }
 
 # we draw the data points new
 ::FR_diagram::draw_curve $VAR(data_range)
}

#package require Tk
#toplevel .


#set data [list 0 0.10827637 0.1 0.12744141 0.2 0.12310791 0.3 0.095184326 0.4 0.050842285 0.5 -0.0033874512 0.6 -0.057525635 0.7 -0.10153198 0.8 -0.12789917 0.9 -0.13189697 1 -0.11227417]
#set data [list 500 0.1 1000 0.1 2000 0.5 2500 0.5 3000 1.0 8000 -1.00]

#::FR_diagram::create -window .c -x_axis_min 500 -x_axis_max 8000 -x_axis_log 1 -data $data
#::FR_diagram::create -window .c -data $data
#::FR_diagram::diagram_title "Falks Diagramm"
