#-----FR_exec based of a code of Frank Pilhofer
#-----and execlog.tcl from Brent B. Welch

# usage:
# ::FR_exec::start command {time in ms | default: infinity} {kill the process after finishing | default: 0}
#
# FR_exec opens a pipe for "command"
# if there no output from "command" after "time" we will break

package provide FR_exec 0.1

namespace eval ::FR_exec \
{
 namespace export start
}


#-----we open a channel
proc ::FR_exec::start {command {time "infinity"} {kill 0}} \
{
 variable exec_id; variable exec_cond; variable exec_channel; variable exec_data
 variable exec_timer; variable exec_pid; variable exec_kill
 
 if {![info exists exec_id]} \
 {
  set exec_id 0
 } else \
 {
  incr exec_id
 }
 
 if {[catch {open "|$command |& cat"} channel]} \
 {
  return [list 1 $channel]
 } else \
 {
  set id $exec_id
  set exec_channel($id) $channel
  set exec_pid($id) [lindex [pid $channel] 0]
  set exec_kill($id) $kill
  set exec_data($id) ""
  set exec_cond($id) 0
  
  if {$time eq "infinity"} \
  {
   set exec_timer($id) $time
  } else \
  {
   set exec_timer($id) [after $time "::FR_exec::exec_break $id"]
  }
  
  fconfigure $channel -blocking 0 
  fileevent $channel readable "::FR_exec::exec_fileevent $id"
 }
 
 vwait ::FR_exec::exec_cond($id)
 
 unset exec_channel($id)
 unset exec_cond($id)
 
 return [list 0 $exec_data($id)]
}


#-----if the channel readable we execute this
proc ::FR_exec::exec_fileevent {id} \
{ 
 variable exec_channel; variable exec_cond; variable exec_data

 if {[eof $exec_channel($id)]} \
 {
  ::FR_exec::exec_stop $id
 } else \
 {
  gets $exec_channel($id) line
  append exec_data($id) $line\n
 }
}


#-----we break after "time"
proc ::FR_exec::exec_break {id} \
{
 variable exec_data
 
 set exec_data($id) ""
 
 ::FR_exec::exec_stop $id
}


#-----the stop command after break or after EOF
proc ::FR_exec::exec_stop {id} \
{
 variable exec_cond; variable exec_channel; variable exec_timer; variable exec_pid
 variable exec_kill
 
 if {$exec_timer($id) ne "infinity"} {after cancel $exec_timer($id)}
 
 fileevent $exec_channel($id) readable ""

 if {[catch {close $exec_channel($id)} err]} \
 {
  error "channel error: $err"
 }
 
 if {$exec_kill($id)} \
 {
  catch {exec kill $exec_pid($id)}
 }
 
 
 set exec_cond($id) 1
}
