package provide FR_general 0.2

namespace eval ::FR_general \
{
 namespace export labelfieldbutton labelfield label_listbox_scrollbar headline topic line output \
 TextwidgetWithScrollbars padframe packhelp_1 packhelp_2 CenterWindow window_maximise \
 ButtonsWithSameWidth scrollwindow_refresh scrollwindow MenuBinding \
 makenormal makedisable
}


#---a line with a label, a entry field and a button
#
proc ::FR_general::labelfieldbutton {window row labeltext buttontext} \
{
 label .$window.label$row -pady 10 -text $labeltext
 entry .$window.entry$row -state normal -width 25 -background white
 button .$window.button$row -text $buttontext

 grid .$window.label$row -row $row -column 0 -sticky w
 grid .$window.entry$row -row $row -column 1 -sticky ew
 grid .$window.button$row -row $row -column 2 -sticky ew

 grid columnconfigure .$window 1 -weight 1
}


#---a line with a label and a entry field
#
proc ::FR_general::labelfield {window row labeltext} \
{
 label .$window.label$row -pady 10 -text $labeltext
 entry .$window.entry$row -state normal -background white

 grid .$window.label$row -row $row -column 0 -sticky w
 grid .$window.entry$row -row $row -column 1 -columnspan 2 -sticky ew

 grid columnconfigure .$window 1 -weight 1
}


#---a listbox with label and y-scrollbar
#
proc ::FR_general::label_listbox_scrollbar {label_widget listbox_widget scrollbar_widget row} \
{
 grid $label_widget -row $row -column 0 -sticky e
 grid $listbox_widget -row $row -column 2 -sticky ew
 grid $scrollbar_widget -row $row -column 3 -sticky nsew
}


#---a main topic line
#
proc ::FR_general::headline {window row labeltext columnspan {column 0}} \
{
 label .$window.label$row -pady 20 -foreground blue -text $labeltext
 grid .$window.label$row -row $row -column $column -columnspan $columnspan -sticky w
}


#---a topic line
#
proc ::FR_general::topic {window row labeltext columnspan {column 0}} \
{
 label .$window.label$row -pady 10 -text $labeltext
 grid .$window.label$row -row $row -column $column -columnspan $columnspan -sticky w
}


#---a line to separate rows
#
proc ::FR_general::line {window row columnspan} \
{
 frame .$window.frame$row -height 2 -borderwidth 1 -relief sunken
 grid .$window.frame$row -row $row -column 0 -columnspan $columnspan -sticky ew
}


#---a output text line with scrollbar
#
proc ::FR_general::output {window row TEXT columnspan} \
{
 text .$window.output$row -height 1 -wrap none -state normal -background white -xscrollcommand ".$window.xsbar$row set"
 scrollbar .$window.xsbar$row -orient horizontal -command ".$window.output$row xview"
 grid .$window.output$row -row $row -column 0 -columnspan $columnspan -sticky ew
 grid .$window.xsbar$row -row [expr {$row+1}] -column 0 -columnspan $columnspan -sticky nsew
 .$window.output$row insert end $TEXT
 .$window.output$row configure -state disabled
}

#---a text-widget with scrollbars
#
proc ::FR_general::TextwidgetWithScrollbars {Path {State normal} {Width 50} {Height 12} {Wrap word}} \
{
 set Path [string trimleft $Path .]
 
 text .$Path.output -width $Width -height $Height -wrap $Wrap -background white -state $State -xscrollcommand ".$Path.xsbar set" -yscrollcommand ".$Path.ysbar set"
 scrollbar .$Path.xsbar -orient horizontal -command ".$Path.output xview"
 scrollbar .$Path.ysbar -orient vertical -command ".$Path.output yview"

 grid .$Path.output .$Path.ysbar -sticky nsew
 grid .$Path.xsbar -sticky nsew
 grid columnconfigure .$Path 0 -weight 1
 grid rowconfigure .$Path 0 -weight 1
}


#---a frame to expand a window managed by grid
#
proc ::FR_general::padframe {window row columnspan {column 0}} \
{ 
 frame .$window.padframe$row -relief flat
  
 grid .$window.padframe$row -row $row -column $column -columnspan $columnspan -sticky news
 grid rowconfigure .$window $row -weight 1
}


#---pack some widgets
#
proc ::FR_general::packhelp_1 {args} \
{
 # packhelp_1  {row widget1 column sticky widget2 column sticky...}
 
 set param_per_widget 3
 set widget_count [expr {([llength $args] - 1) / $param_per_widget}]
 set row [lindex $args 0]

 for {set x 1} {$x <= $widget_count} {incr x} \
 {
  set index [expr {${param_per_widget} * ($x-1) + 1}] 
  set widget [lindex $args $index]
  set column [lindex $args [incr index]]
  set sticky [lindex $args [incr index]]
  
  grid $widget -row $row -column $column -sticky $sticky
 }
}


proc ::FR_general::packhelp_2 {args} \
{
 # packhelp_2 {row widget1 column columnspan sticky widget2 column columnspan sticky...}
 set param_per_widget 4
 set widget_count [expr {([llength $args] - 1) / $param_per_widget}]
 set row [lindex $args 0]

 for {set x 1} {$x <= $widget_count} {incr x} \
 {
  set index [expr {${param_per_widget} * ($x-1) + 1}]
  set widget [lindex $args $index]
  set column [lindex $args [incr index]]
  set columnspan [lindex $args [incr index]]
  set sticky [lindex $args [incr index]]
  grid $widget -row $row -column $column -columnspan $columnspan -sticky $sticky
 }
}


#---We make a window to a child of a parent window and put it central on the parent window
#---Args: {Child window in the foreground} {parent window in the background} {make it transient to the parent} {desired X of the child window} {desired Y of the child window}
#
proc ::FR_general::CenterWindow {Window {Parent .} {Transient 1} {WindowX 1} {WindowY 1}} \
{
 update
 
 regexp {([0-9]+)x([0-9]+)\+([\-0-9]+)\+([\-0-9]+)} [winfo geometry $Parent] All ParentWidth ParentHeight ParentX ParentY

 #---"All" doesn't exist if "Parent" is minimized
 if {[info exists All]} \
 {
   unset All
 } else \
 {
   # if Parent minimized we place Window in the left-top corner of the screen
   set ParentWidth [winfo reqwidth $Window]
   set ParentHeight [winfo reqheight $Window]
   set ParentX 0
   set ParentY 0
 }
 
 if { ($WindowX == 1) && ($WindowY == 1)} \
 {
   set WindowX [winfo reqwidth $Window]
   set WindowY [winfo reqheight $Window]
 }
 
 set X [expr {($ParentWidth-$WindowX)/2+$ParentX}]
 set Y [expr {($ParentHeight-$WindowY)/2+$ParentY}]
 
 set ScreenHeight [winfo screenheight $Window]
 set ScreenWidth [winfo screenwidth $Window]
  
 # if X or Y are out of the screen -> we correct this
 if {[expr {${WindowX} + $X}] > $ScreenWidth} {set X [expr {${ScreenWidth} - ${WindowX}}]}
 if {[expr {${WindowY} + $Y}] > $ScreenHeight} {set Y [expr {${ScreenHeight} - ${WindowY}}]}
 if {$X < 0} {set X 0}
 if {$Y < 0} {set Y 0}
 
 wm geometry $Window ${WindowX}x$WindowY+$X+$Y
 
 if {$Transient} \
 { 
   wm transient $Window $Parent
 }
}

#---We maximise all handed over windows
#---Args: {Window(s)-path(s)} {0...1 of the screenwith} {0...1 of the screenheight}
#
proc ::FR_general::window_maximise {WINDOWS {PARTX 1} {PARTY 1}} \
{
 update
 
 set SCREENX [winfo screenwidth .]
 set SCREENY [winfo screenheight .]
 
 set SCREENX [expr {round($SCREENX*$PARTX)}]
 set SCREENY [expr {round($SCREENY*$PARTY)}]
 
 foreach WINDOW $WINDOWS \
 {
  wm geometry $WINDOW ${SCREENX}x${SCREENY}+0+0
 }
}

#---we restore all handed over windows
#
proc ::FR_general::window_position_restore {WINDOWS POSITIONS} \
{
 update
 
 foreach WINDOW $WINDOWS POSITION $POSITIONS \
 {
  set ALL ""
  regexp {([0-9]+)x([0-9]+)\+([\-0-9]+)\+([\-0-9]+)} $POSITION ALL WITDTH HEIGHT X Y
  
  if {$ALL ne ""} \
  {
   if {$X < 0} {set X 0}
   if {$Y < 0} {set Y 0}
 
   wm geometry $WINDOW +$X+$Y
  }
 }
}


#---we give back the right width for two or more buttons after translation
#
proc ::FR_general::ButtonsWithSameWidth {ButtonTextList} \
{
  set Width 0
  set ReturnValue ""
  
  foreach Arg $ButtonTextList \
  {
    set Length [string length "$Arg"]
    
    if {$Length > $Width} {set Width $Length}
    
    lappend ReturnValue $Arg
  }
 
  lappend ReturnValue $Width
 
 
  return $ReturnValue
}


#---refresh the width and the height of the scrollable canvas window
#
proc ::FR_general::scrollwindow_refresh {window} \
{
 update
   
 set WIDTH [winfo width .$window.scrollwindow]
 set HEIGHT_CANVAS [winfo height .$window.scrollwindow]
 set HEIGHT [winfo reqheight .$window.scrollwindow.content]
 
 #---if the content of the canvas window smaller then the canvas window itself we block the scrollbars function with this
 if {$HEIGHT <= $HEIGHT_CANVAS} \
  {
   set BBOX [list 0 0 $WIDTH $HEIGHT_CANVAS]
  } else \
  {
   set BBOX [list 0 0 $WIDTH $HEIGHT]
  }
 
 #---set the width of the content to the width of the canvas
 .$window.scrollwindow itemconfigure tag1 -width $WIDTH 
  
 #---set the scrollregion
 .$window.scrollwindow configure -scrollregion $BBOX
}


#-----a scrollwindow with vertical scrollbars-----
proc ::FR_general::scrollwindow {WINDOW SCROLLBARS} \
{
 scrollbar .$WINDOW.sbary -command ".$WINDOW.scrollwindow yview" -orient vertical
 
 canvas .$WINDOW.scrollwindow -yscrollcommand ".$WINDOW.sbary set" -borderwidth 0 -relief flat
 
 pack .$WINDOW.sbary -side right -fill y
 pack .$WINDOW.scrollwindow -side left -fill both -expand yes
  
 frame .$WINDOW.scrollwindow.content -borderwidth 0
 
 #---tag for width setting in "scrollwindow_refresh"
 .$WINDOW.scrollwindow create window 0 0 -anchor nw -window .$WINDOW.scrollwindow.content -tags tag1

 bind .$WINDOW <Configure> "scrollwindow_refresh $WINDOW"
 
 return $WINDOW.scrollwindow.content
}


#---proc to bind the menu items to a keyboard shortcut
proc ::FR_general::MenuBinding {menupath Position} \
{
  $menupath invoke $Position
  $menupath unpost
  event generate $menupath <KeyPress-Escape>
}


#-----set some widgets to normal-----
proc ::FR_general::makenormal {args} \
{
 foreach widget $args \
 {
  $widget configure -state normal
 }
}


#-----set some widgets to disable-----
proc ::FR_general::makedisable {args} \
{
 foreach widget $args \
 {
  $widget configure -state disable
 }
}