#----------the text widget as message window
#
# ::FR_msgWindow::create
# -window "a frame in which we can insert the message widget"
# 
# ::FR_msgWindow::add
# -window (path to the msg-window) if no path was overtaken: default is the last created msg-window
# -format (heading | body | info | error) default: body
# -msg (message)

package provide FR_msgWindow 0.1


namespace eval ::FR_msgWindow \
{
 namespace export create add
}


proc ::FR_msgWindow::create {args} \
{
 variable msgwindow; variable msgwindow_path
 
 set msgwindow_path ""
 
 for {set i 0} {$i < [llength $args]} {incr i 2} \
 {
  set arg [lindex $args $i]
  switch -glob -- $arg \
  {
   -window       {set msgwindow_path [lindex $args [expr {$i+1}]]} \
   --            {break} \
   *             {error "unknown option: $arg"}
  }
 }

   
 if { $msgwindow_path eq "" } \
 {
  error "You must set a window name for your msgWindow with \"-window\""
 }
 
 set msgwindow_path [string trimleft $msgwindow_path .]
 set msgwindow($msgwindow_path,PATH) $msgwindow_path
 
 
 label .$msgwindow_path.topic -text [msgcat::mc "Process messages:"]
 button .$msgwindow_path.clear_button -text [msgcat::mc "Clear"] -command [list ::FR_msgWindow::clear $msgwindow_path.text]

 text .$msgwindow_path.text -wrap none -background white -state disabled -xscrollcommand ".$msgwindow_path.xsbar set" -yscrollcommand ".$msgwindow_path.ysbar set"
 scrollbar .$msgwindow_path.xsbar -orient horizontal -command ".$msgwindow_path.text xview"
 scrollbar .$msgwindow_path.ysbar -orient vertical -command ".$msgwindow_path.text yview"
 
 grid .$msgwindow_path.topic -row 0 -column 0 -sticky w -pady {0 5}
 grid .$msgwindow_path.clear_button -row 0 -column 2 -columnspan 2 -sticky e -pady {0 5}
 grid .$msgwindow_path.text -row 1 -column 0 -columnspan 3 -sticky nsew
 grid .$msgwindow_path.ysbar -row 1 -column 3 -sticky ns
 grid .$msgwindow_path.xsbar -row 2 -column 0 -columnspan 3 -sticky ew
 
 grid columnconfigure .$msgwindow_path 1 -weight 1
 grid rowconfigure .$msgwindow_path 1 -weight 1
 
 # we configure the text tags
 .$msgwindow_path.text tag configure "heading" -foreground blue -underline yes -lmargin1 0.1i
 .$msgwindow_path.text tag configure "body" -foreground black -lmargin1 0.2i
 .$msgwindow_path.text tag configure "info" -foreground black -lmargin1 0.1i
 .$msgwindow_path.text tag configure "error" -foreground red -lmargin1 0.1i
}


#---add some informations to the msg window
proc ::FR_msgWindow::add {args}\
{
 variable msgwindow; variable msgwindow_path
 
 set WINDOW ""
 set FORMAT ""
 set MESSAGE ""
 
 for {set i 0} {$i < [llength $args]} {incr i 2} \
 {
  set arg [lindex $args $i]
  switch -glob -- $arg \
  {
   -window       {set WINDOW [lindex $args [expr {$i+1}]]} \
   -format       {set FORMAT [lindex $args [expr {$i+1}]]} \
   -msg          {set MESSAGE [lindex $args [expr {$i+1}]]} \
   --            {break} \
   *             {error "unknown option: $arg"}
  }
 }
 
 # if now path was overtaken we assume that you have only one msg window
 # or you mean the msg window which was created recently
 if {$WINDOW eq ""} {set WINDOW $msgwindow_path}
 
 if {$FORMAT eq ""} {set FORMAT "body"}
 
 .$msgwindow($WINDOW,PATH).text configure -state normal
 .$msgwindow($WINDOW,PATH).text insert end $MESSAGE $FORMAT
 .$msgwindow($WINDOW,PATH).text insert end "\n"
 update
 .$msgwindow($WINDOW,PATH).text see end
 update
 .$msgwindow($WINDOW,PATH).text configure -state disabled
}

#---we clear the msg window
proc ::FR_msgWindow::clear {path} \
{
 .$path configure -state normal
 .$path delete 1.0 end
 update
 .$path configure -state disabled
}